import java.io.IOException;

import javax.microedition.io.StreamConnection;
import javax.microedition.lcdui.Display;

import protocol.MusicPlayer;
import protocol.ProtocolStatus;
import protocol.StatusUpdateListener;

import bemused.BemusedProtocol;

/*
 * Created on May 24, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */

/**
 * @author fred
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class Controller implements StatusUpdateListener {
	Main owner;
	SearchForm searchForm;
	BrowseForm browseForm;
	PlaylistForm playlistForm;
	ControlForm controlForm;
	JumpForm jumpForm;
	Settings settings;
	StreamConnection currentConnection;
	BemusedProtocol protocol;
	
	public Controller(Main owner) {
		this.owner = owner;
		protocol = new BemusedProtocol(Display.getDisplay(owner));
		protocol.registerStatusUpdateListener(this);
		settings = new Settings();
		searchForm = new SearchForm(this);
		browseForm = new BrowseForm(this);
		playlistForm = new PlaylistForm(this);
		controlForm = new ControlForm(this);
		jumpForm = new JumpForm(this);
	}

	Settings getSettings() {
		return settings;
	}

	public void start() {
		Display.getDisplay(owner).setCurrent(searchForm);		
	}

	public void showPlaylist() {
		protocol.refreshPlaylist();
		Display.getDisplay(owner).setCurrent(playlistForm);				
	}
	
	public void showBrowser() {
		showBrowser(true);
	}
	
	public void showBrowser(boolean refresh) {
		if (refresh) protocol.fileBrowser.readDir();
		Display.getDisplay(owner).setCurrent(browseForm);				
	}
	
	public void showController() {
		Display.getDisplay(owner).setCurrent(controlForm);				
	}
	
	public void exit() {
		owner.destroyApp(false);
		owner.notifyDestroyed();
	}
	
	public void showJumper() {
		Display.getDisplay(owner).setCurrent(jumpForm);	
	}

	void destroy() {
		disconnect();
		settings.save();
		if (protocol != null) {
			//protocol.destroy();
			protocol = null;
		}
	}
	
	void connectTo(String url, String name) {
		settings.setLastUsedDevice(url, name);
		settings.save();
		protocol.setConnectionURL(url);
	}
	
	void disconnect() {	
		protocol.closeConnection();
	}
	
	/* (non-Javadoc)
	 * @see bemused.StatusUpdateListener#bemusedStatusChanged(bemused.BemusedProtocol)
	 */
	public void bemusedStatusChanged(ProtocolStatus status) {
		Display display = Display.getDisplay(owner);
		if (status.connected && (display.getCurrent() == searchForm)) {
			display.setCurrent(controlForm);
		}
		else if (status.connected == false && (display.getCurrent() != searchForm)) {
			display.setCurrent(searchForm);
		}
		
		BemusedProtocol.PlaybackStatus s = protocol.status();
		controlForm.setStatus(s);
		
		playlistForm.updatePlaylist(protocol.currentTrackNum(), protocol.playlist());
		browseForm.updateDirList(protocol.fileBrowser.dirs(), protocol.fileBrowser.files(), protocol.fileBrowser.dirChanged());
	}

	MusicPlayer getPlayer() {
		return protocol;
	}
	
}
